%%%%%%% icon_mode.sl -- simple mode for editing icon programs %%%%%%%%%%%%%%%%%
%%
%% This is a simple icon mode.  It does not define any form of indentation
%% style.  Rather, it simply implements a highlighting scheme.
%%
%% Copyright  1998 Klaus Alexander Seistrup @ Magnetic Ink, Ndola, ZAMBIA.
%%
%% @(#) Author  : 1998 Klaus Alexander Seistrup <mutende@magnetic-ink.dk>
%% @(#) Created : Mon Dec 28 12:12 1998
%% @(#) Version : 0
%% @(#) Keywords: icon editing language mode
%%
%% @(#) $Id: icon_mode.sl,v 1.2 1998/12/28 11:28:43 mutende Exp $
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITH-
%% OUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
%%
%%   "Now that all has been heard; here is the conclusion of the matter:
%%    Fear God and keep his commandments, for this is the whole duty of
%%    man.  For God will bring every deed into judgment, including every
%%    hidden thing, whether it is good or evil."
%%                                                Ecclesiastes 12:13-14
%%
%% Let the show begin: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
$1 = "Icon-Mode";

create_syntax_table ($1);

define_syntax ("#", "", '%', $1);		% Comments
define_syntax ('$', '#', $1);			% Preprocessor
define_syntax ("([{", ")]}", '(', $1);		% Braces
define_syntax ('"', '"', $1);			% String delimiter
define_syntax ('\'', '\'', $1);			% Character delimiter
define_syntax ('\\', '\\', $1);			% Quote character
define_syntax ("0-9A-Z_a-z", 'w', $1);		% Words
define_syntax ("-+0-9A-Fa-f.XxLl", '0', $1);	% Numbers
define_syntax (",.:;", ',', $1);		% Delimiters
define_syntax ("%-+/&*=<>|!?~^", '+', $1);	% Operators

%% Icon built-in keywords #1
pop (define_keywords_n ($1, "bydoifofto", 2, 0));
pop (define_keywords_n ($1, "endnot", 3, 0));
pop (define_keywords_n ($1, "caseelsefaillinknextthen", 4, 0));
pop (define_keywords_n ($1, "breakeverylocaluntilwhile", 5, 0));
pop (define_keywords_n ($1, "createglobalrecordrepeatreturnstatic", 6, 0));
pop (define_keywords_n ($1, "defaultinitialsuspend", 7, 0));
pop (define_keywords_n ($1, "invocableprocedure", 9, 0));

%% Icon built-in keywords #2
pop (define_keywords_n ($1, "exy", 1, 1));
pop (define_keywords_n ($1, "pi", 2, 1));
pop (define_keywords_n ($1, "colphiposrow", 3, 1));
pop (define_keywords_n ($1, "csetdatedumpfailfilehostlinemainmetanulltime", 4, 1));
pop (define_keywords_n ($1, "asciiclockerrorinputlcaseldraglevelmdragrdragshifttraceucase", 5, 1));
pop (define_keywords_n ($1, "columndigitserroutlpressmpressoutputrandomresizerpresssourcewindow", 6, 1));
pop (define_keywords_n ($1, "controlcurrentlettersregionsstoragesubjectversion", 7, 1));
pop (define_keywords_n ($1, "datelinefeaturesintervallreleasemreleaseprognamerrelease", 8, 1));
pop (define_keywords_n ($1, "allocatederrortexteventcode", 9, 1));
pop (define_keywords_n ($1, "errorvalueeventvalue", 10, 1));
pop (define_keywords_n ($1, "collectionserrornumbereventsource", 11, 1));

%%
define icon_mode_version ()
{
  flush ("@(#) $Id: icon_mode.sl,v 1.2 1998/12/28 11:28:43 mutende Exp $");
}

%%
define icon_insert_comment ()
{
  $1 = "# ";

  insert ($1);

  while (down (2) == 2)
    {
      go_up_1 ();
      bol ();
      insert ($1);
    }
}

%%
define icon_uncomment_region ()
{
  $1 = "# ";

  push_spot ();
  narrow ();
  bob ();
  go_down_1 ();

  do
    {
      go_up_1 ();
      bol ();

      if (looking_at ($1))
	deln (2);
    }
  while (down (2) == 2);

  widen ();
  pop_spot ();
}

%%
define icon_comment_region ()
{
  if (-1 == prefix_argument (-1))
    {
      push_spot ();
      narrow ();
      bob ();
      icon_insert_comment ();
      widen ();
      pop_spot ();
    }
  else
    icon_uncomment_region ();
}

%% The icon keymap
!if (keymap_p ($1))
{
  make_keymap ($1);

  definekey ("newline_and_indent", "\r", $1);
  definekey ("indent_line", "\t", $1);
}

%% The icon mode
define icon_mode ()
{
  variable mode = "Icon";
  variable icon_map = "Icon-Mode";

  setmode (mode, 4);			% Language mode
  use_syntax_table (icon_map);
  use_keymap (icon_map);
  % set_buffer_hook ("indent_hook", "icon_indent_line");
  % set_buffer_hook ("newline_indent_hook", "icon_newline_and_indent");
  runhooks ("icon_mode_hook");

  % This is called after the hook to give the hook a chance to load
  % the abbrev table.
  if (abbrev_table_p (icon_map))
    use_abbrev_table (icon_map);
}

%% EOF
